@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2006 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.cc,v 1.117 2005/12/08 13:50:06 lav Exp $ */
+/* $Id: CmdExec.cc,v 1.120 2006/04/19 09:48:02 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -37,6 +37,7 @@
 #include "QueueFeeder.h"
 #include "LocalDir.h"
 #include "ConnectionSlot.h"
+#include "DummyProto.h"
 
 #define RL_PROMPT_START_IGNORE	'\001'
 #define RL_PROMPT_END_IGNORE	'\002'
@@ -63,8 +64,9 @@ static ResDecl
    res_term_status         ("cmd:term-status",  "", 0, 0),
    res_trace		   ("cmd:trace",  "no",	ResMgr::BoolValidate,ResMgr::NoClosure);
 
-CmdExec	 *CmdExec::cwd_owner=0;
-CmdExec	 *CmdExec::chain=0;
+CmdExec	 *CmdExec::cwd_owner;
+CmdExec	 *CmdExec::chain;
+CmdExec  *CmdExec::top;
 
 void  CmdExec::SaveCWD()
 {
@@ -775,7 +777,7 @@ void CmdExec::PrintStatus(int v,const char *prefix)
    }
 }
 
-CmdExec::CmdExec(FileAccess *f,LocalDirectory *c) : SessionJob(f)
+CmdExec::CmdExec(FileAccess *f,LocalDirectory *c) : SessionJob(f?f:new DummyProto)
 {
    // add this to chain
    next=chain;
@@ -803,6 +805,8 @@ CmdExec::CmdExec(FileAccess *f,LocalDirectory *c) : SessionJob(f)
    last_bg=-1;
 
    cwd=c;
+   if(!cwd)
+      SaveCWD();
 
    remote_completion=false;
    long_running=0;
