@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ConnectionSlot.cc,v 1.5 2003/11/20 14:39:10 lav Exp $ */
+/* $Id: ConnectionSlot.cc,v 1.6 2005/08/03 07:12:44 lav Exp $ */
 
 #include <config.h>
 #include "FileAccess.h"
@@ -63,7 +63,7 @@ void ConnectionSlot::Set(const char *n,FileAccess *fa)
    SessionPool::Reuse(s->session);
    s->session=fa->Clone();
 }
-void ConnectionSlot::SetCwd(const char *n,const char *cwd)
+void ConnectionSlot::SetCwd(const char *n,const FileAccess::Path &cwd)
 {
    ConnectionSlot::SlotValue *s=Find(n);
    if(!s)
