@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.h,v 1.61 2005/06/22 08:04:18 lav Exp $ */
+/* $Id: FileCopy.h,v 1.64 2006/02/06 11:00:38 lav Exp $ */
 
 /*
   classes defined here:
@@ -66,6 +66,9 @@ protected:
 
    bool write_allowed;
 
+   char *suggested_filename;
+   bool auto_rename;
+
 public:
    off_t range_start; // NOTE: ranges are implemented only partially. (FIXME)
    off_t range_limit;
@@ -134,6 +137,13 @@ public:
 
    const char *GetDescriptionForLog() { return 0; }
    virtual const char *GetURL() { return 0; }
+
+   const char *GetSuggestedFileName() { return suggested_filename; }
+   void SetSuggestedFileName(const char *f)
+      {
+	 if(f && !suggested_filename)
+	    suggested_filename=xstrdup(f);
+      }
 };
 
 class FileCopy : public SMTask
