@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2000-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2000-2006 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -674,7 +674,7 @@ int Fish::HandleReplies()
 	       SetError(LOGIN_FAILED,_("Password required"));
 	       return MOVED;
 	    }
-	    if(password_sent>0)
+	    if(password_sent>1)
 	    {
 	       SetError(LOGIN_FAILED,_("Login incorrect"));
 	       return MOVED;
@@ -704,8 +704,10 @@ int Fish::HandleReplies()
    recv_buf->Skip(s);
 
    int code=-1;
-   if(s>7 && !memcmp(line,"### ",4) && is_ascii_digit(line[4]))
-      sscanf(line+4,"%3d",&code);
+   if(s>7 && !memcmp(line,"### ",4)) {
+      if(sscanf(line+4,"%3d",&code)!=1)
+	 code=-1;
+   }
 
    DebugPrint("<--- ",line,ReplyLogPriority(code));
    if(code==-1)
