@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Fish.h,v 1.18 2001/12/20 11:14:43 lav Exp $ */
+/* $Id: Fish.h,v 1.21 2005/06/22 08:04:19 lav Exp $ */
 
 #ifndef FISH_H
 #define FISH_H
@@ -42,6 +42,7 @@ class Fish : public NetAccess
 
    state_t state;
    bool received_greeting;
+   int  password_sent;
 
    void Init();
 
@@ -52,8 +53,6 @@ class Fish : public NetAccess
 
    IOBuffer *send_buf;
    IOBuffer *recv_buf;
-   bool recv_buf_suspended;
-
    PtyShell *ssh;
 
    void Disconnect();
@@ -152,8 +151,8 @@ public:
 
    bool NeedSizeDateBeforehand() { return true; }
 
-   void Suspend();
-   void Resume();
+   void SuspendInternal();
+   void ResumeInternal();
 
    void Cleanup();
    void CleanupThis();
@@ -162,7 +161,7 @@ public:
 class FishDirList : public DirList
 {
    FileAccess *session;
-   Buffer *ubuf;
+   IOBuffer *ubuf;
    char *pattern;
 
 public:
@@ -171,8 +170,8 @@ public:
    const char *Status();
    int Do();
 
-   void Suspend();
-   void Resume();
+   void SuspendInternal();
+   void ResumeInternal();
 };
 
 class FishListInfo : public GenericParseListInfo
