@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FtpListInfo.cc,v 1.58 2005/05/12 10:34:31 lav Exp $ */
+/* $Id: FtpListInfo.cc,v 1.61 2006/02/06 10:57:25 lav Exp $ */
 
 #include <config.h>
 #include "FtpListInfo.h"
@@ -480,7 +480,8 @@ FileInfo *ParseFtpLongList_EPLF(char *line,int *err,const char *)
 	    break;
 	 case 'u':
 	    if(scan[1]=='p')  // permissions.
-	       sscanf(scan+2,"%o",&perms);
+	       if(sscanf(scan+2,"%o",&perms)!=1)
+		  perms=-1;
 	    break;
 	 default:
 	    name=0;
@@ -794,7 +795,8 @@ FileInfo *ParseFtpLongList_MLSD(char *line,int *err,const char *)
       }
       if(!strncasecmp(tok,"UNIX.mode=",10))
       {
-	 sscanf(tok+10,"%o",&perms);
+	 if(sscanf(tok+10,"%o",&perms)!=1)
+	    perms=-1;
 	 continue;
       }
       if(!strncasecmp(tok,"UNIX.owner=",11))
