@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1999-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2006 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.199 2005/08/04 10:10:29 lav Exp $ */
+/* $Id: Http.cc,v 1.207 2006/03/31 09:42:15 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -98,6 +98,8 @@ void Http::Init()
    no_cache_this=false;
    no_cache=false;
 
+   use_propfind_now=true;
+
    hftp=false;
    https=false;
    use_head=true;
@@ -233,6 +235,7 @@ void Http::Close()
    array_send=0;
    no_cache_this=false;
    no_ranges=false;
+   use_propfind_now=QueryBool("use-propfind",hostname);
    special=HTTP_NONE;
    xfree(special_data);
    special_data=0;
@@ -436,33 +439,31 @@ bool Http::ModeSupported()
 
 void Http::DirFile(char *path_base,const char *ecwd,const char *efile)
 {
+   if(!strcmp(ecwd,"~") && !hftp)
+      ecwd="";
+   const char *sep=(last_char(ecwd)=='/'?"":"/");
+   if(efile[0]==0)
+      sep="";
+   const char *pre=(ecwd[0]=='/'?"":"/");
    if(efile[0]=='/')
       strcpy(path_base,efile);
    else if(efile[0]=='~')
       sprintf(path_base,"/%s",efile);
-   else if(ecwd[0]==0 || ((ecwd[0]=='/' || (!hftp && ecwd[0]=='~')) && ecwd[1]==0))
-      sprintf(path_base,"/%s",efile);
-   else if(ecwd[0]=='~' && efile[0])
-      sprintf(path_base,"/%s/%s",ecwd,efile);
-   else if(efile[0])
-      sprintf(path_base,"%s/%s",ecwd,efile);
    else
-      strcpy(path_base,ecwd);
+      sprintf(path_base,"%s%s%s%s",pre,ecwd,sep,efile);
 
-   if(path_base[1]=='~' && path_base[2]=='/')
+   if(path_base[1]=='~' && path_base[2]==0)
+      path_base[1]=0;
+   else if(path_base[1]=='~' && path_base[2]=='/')
       memmove(path_base,path_base+2,strlen(path_base+2)+1);
-   else if(hftp && path_base[1]!='~')
-   {
-      // root directory in ftp urls needs special encoding. (/%2Fpath)
-      memmove(path_base+4,path_base+1,strlen(path_base+1)+1);
-      memcpy(path_base+1,"%2F",3);
-   }
 }
 
 void Http::SendRequest(const char *connection,const char *f)
 {
    char *efile=string_alloca(strlen(f)*3+1);
    url::encode_string(f,efile,URL_PATH_UNSAFE);
+   if(mode==CHANGE_DIR && new_cwd && new_cwd->url)
+      efile=new_cwd->url+url::path_index(new_cwd->url);
    char *ecwd;
    int efile_len;
 
@@ -473,8 +474,14 @@ void Http::SendRequest(const char *connection,const char *f)
    }
    else
    {
-      ecwd=string_alloca(strlen(cwd)*3+1);
+      ecwd=string_alloca(strlen(cwd)*3+3+1);
       url::encode_string(cwd,ecwd,URL_PATH_UNSAFE);
+      if(hftp && ecwd[0]=='/' && ecwd[1]!='~')
+      {
+	 // root directory in ftp urls needs special encoding. (/%2Fpath)
+	 memmove(ecwd+4,ecwd+1,strlen(ecwd+1)+1);
+	 memcpy(ecwd+1,"%2F",3);
+      }
    }
    bool add_slash=true;
    if(cwd.is_file)
@@ -610,7 +617,17 @@ void Http::SendRequest(const char *connection,const char *f)
       if(last_char(efile)!='/' && add_slash)
 	 strcat(efile,"/");
       if(mode==CHANGE_DIR)
-	 SendMethod("HEAD",efile);
+      {
+	 if(use_propfind_now)
+	 {
+	    SendMethod("PROPFIND",efile);
+	    Send("Depth: 0\r\n"); // no directory listing required
+	    Send("Content-Type: text/xml\r\n");
+	    Send("Content-Length: %d\r\n",strlen(allprop));
+	 }
+	 else
+	    SendMethod("HEAD",efile);
+      }
       else if(mode==LONG_LIST)
 	 goto retrieve;
       else if(mode==MAKE_DIR)
@@ -668,7 +685,7 @@ void Http::SendRequest(const char *connection,const char *f)
 	 Send("%s",special_data);
       entity_size=NO_SIZE;
    }
-   else if(mode==MP_LIST)
+   else if(mode==MP_LIST || (mode==CHANGE_DIR && use_propfind_now))
    {
       Send("%s",allprop);
    }
@@ -732,6 +749,8 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       long long bs=0;
       if(1!=sscanf(value,"%lld",&bs))
 	 return;
+      if(bs<0) // try to workaround broken servers
+	 bs+=0x100000000LL;
       body_size=bs;
       if(pos==0 && mode!=STORE && mode!=MAKE_DIR)
 	 entity_size=body_size;
@@ -794,9 +813,10 @@ void Http::HandleHeaderLine(const char *name,const char *value)
    {
       keep_alive=true;
       const char *m=strstr(value,"max=");
-      if(m)
-	 sscanf(m+4,"%d",&keep_alive_max);
-      else
+      if(m) {
+	 if(sscanf(m+4,"%d",&keep_alive_max)!=1)
+	    keep_alive=false;
+      } else
 	 keep_alive_max=100;
       return;
    }
@@ -1034,7 +1054,7 @@ int Http::Do()
 	    return MOVED;
       }
 
-      if(!ReconnectAllowed())
+      if(!resolver && mode!=CONNECT_VERIFY && !ReconnectAllowed())
 	 return m;
 
       if(https)
@@ -1047,6 +1067,9 @@ int Http::Do()
       if(mode==CONNECT_VERIFY)
 	 return m;
 
+      if(!ReconnectAllowed())
+	 return m;
+
       if(!NextTry())
       	 return MOVED;
 
@@ -1482,6 +1505,13 @@ int Http::Do()
 		  if(!xstrcmp(last_method,"MKCOL"))
 		     ResMgr::Set("http:use-mkcol",hostname,"no");
 	       }
+	       if(mode==CHANGE_DIR && !xstrcmp(last_method,"PROPFIND"))
+	       {
+		  use_propfind_now=false;
+		  try_time=0;
+		  Disconnect();
+		  return MOVED;
+	       }
 	       code=NOT_SUPP;
 	       closure=last_method;
 	    }
