@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: HttpDir.cc,v 1.88 2005/08/09 06:40:40 lav Exp $ */
+/* $Id: HttpDir.cc,v 1.92 2006/02/06 10:57:26 lav Exp $ */
 
 #include <config.h>
 #include "ascii_ctype.h"
@@ -688,6 +688,9 @@ static int parse_html(const char *buf,int len,bool eof,Buffer *list,
    if(more-less<3)
       return tag_len;   // too small
 
+   if(less[1]=='/' || less[1]=='!')
+      return tag_len;
+
    int max_link_len=more-less+1+2;
    if(base_href && *base_href)
       max_link_len+=strlen(*base_href)+1;
@@ -745,25 +748,29 @@ static int parse_html(const char *buf,int len,bool eof,Buffer *list,
 
    decode_amps(link_target);  // decode all &amp; and similar
 
+   if(hftp)
+   {
+      // workaround proxy bugs.
+      char *t=strstr(link_target,";type=");
+      if(t && t[6] && t[7]=='/' && t[8]==0)
+	 *t=0;
+      char *p=link_target+url::path_index(link_target);
+      if(p[0]=='/' && p[1]=='/')
+      {
+	 memmove(p+4,p+2,strlen(p+2)+1);
+	 memcpy(p+1,"%2F",3);
+      }
+   }
+
+   Log::global->Format(10,"Found tag %s, link_target=%s\n",tag_scan->tag,link_target);
+
    if(!strcasecmp(tag_scan->tag,"base"))
    {
       if(base_href)
       {
 	 xfree(*base_href);
 	 *base_href=xstrdup(link_target,+2);
-	 if(hftp)
-	 {
-	    // workaround apache proxy bugs.
-	    char *t=strstr(*base_href,";type=");
-	    if(t && t[6] && t[7]=='/' && t[8]==0)
-	       *t=0;
-	    char *p=*base_href+url::path_index(*base_href);
-	    if(p[0]=='/' && p[1]=='/')
-      	    {
-	       memmove(p+4,p+2,strlen(p+2)+1);
-	       memcpy(p+1,"%2F",3);
-	    }
-	 }
+	 Log::global->Format(10,"Found base href=%s\n",*base_href);
       }
       return tag_len;
    }
@@ -875,12 +882,18 @@ parse_url_again:
    if(info.is_directory && link_len>1)
       link_target[--link_len]=0;
 
+   FileAccess::Path::Optimize(link_target+(link_target[0]=='/' && link_target[1]=='~'));
+   link_len=strlen(link_target);
+
    if(prefix)
    {
-      const char *p_path=prefix->path;
-      if(p_path==0)
-	 p_path="~";
+      const char *p_path_c=prefix->path;
+      if(p_path_c==0)
+	 p_path_c="~";
+      char *p_path=alloca_strdup(p_path_c);
       int p_len=strlen(p_path);
+      if(p_len>1 && p_path[p_len-1]=='/')
+	 p_path[--p_len]=0;
       if(p_len==1 && p_path[0]=='/' && link_target[0]=='/')
       {
 	 if(link_len>1)
@@ -934,6 +947,13 @@ parse_url_again:
       type[0]=';';
    }
 
+   if(link_len==0)
+   {
+      strcpy(link_target,".");
+      link_len=1;
+      info.is_directory=true;
+   }
+
    bool show_in_list=true;
    if(icon && (link_target[0]=='/' || link_target[0]=='~'))
       show_in_list=false;  // makes apache listings look better.
@@ -1075,7 +1095,8 @@ parse_url_again:
 	 if(strspn(info.size_str,"0123456789")==strlen(info.size_str))
 	 {
 	    long long size_ll=0;
-	    sscanf(info.size_str,"%lld",&size_ll);
+	    if(sscanf(info.size_str,"%lld",&size_ll)!=1)
+	       size_ll=0;
 	    info.size=size_ll;
 	 }
       }
