@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1998 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Job.cc,v 1.29 2003/12/17 12:59:37 lav Exp $ */
+/* $Id: Job.cc,v 1.37 2006/04/19 09:48:03 lav Exp $ */
 
 #include <config.h>
 #include <stdlib.h>
@@ -53,7 +53,8 @@ void  Job::AllocJobno()
 Job::~Job()
 {
    xfree(waiting);
-
+   waiting=0;
+   waiting_num=0;
    // reparent or kill children (hm, that's sadistic)
    {
       for(Job *scan=chain; scan; )
@@ -61,22 +62,19 @@ Job::~Job()
 	 if(scan->parent==this)
 	 {
 	    if(scan->jobno!=-1 && this->parent)
-	    {
 	       scan->parent=this->parent;
-	       scan=scan->next;
-	    }
-	    else if(!scan->deleting)
-	    {
-	       Delete(scan);
-	       scan=chain;
-	    }
 	    else
-	       scan=scan->next;
+	    {
+	       scan->parent=0;
+	       scan->deleting=true;
+	    }
 	 }
-	 else
-	    scan=scan->next;
+	 scan=scan->next;
       }
    }
+   // if parent waits for the job, make it stop
+   if(parent)
+      parent->RemoveWaiting(this);
    // now, delete the job from the list
    {
       for(Job **scan=&chain; *scan; scan=&(*scan)->next)
@@ -187,35 +185,42 @@ public:
    int	 ExitCode() { return 255; }
 };
 
+void Job::Kill(Job *j)
+{
+   if(j->parent && j->parent->WaitsFor(j))
+   {
+      // someone waits for termination of this job, so
+      // we have to simulate normal death...
+      Job *r=new KilledJob();
+      r->parent=j->parent;
+      r->cmdline=j->cmdline,j->cmdline=0;
+      r->waiting=j->waiting,j->waiting=0;
+      r->waiting_num=j->waiting_num,j->waiting_num=0;
+      j->parent->ReplaceWaiting(j,r);
+   }
+   assert(FindWhoWaitsFor(j)==0);
+   j->DeleteLater();
+}
+
 void Job::Kill(int n)
 {
    Job *j=FindJob(n);
    if(j)
-   {
-      if(j->parent && j->parent->WaitsFor(j))
-      {
-	 // someone waits for termination of this job, so
-	 // we have to simulate normal death...
-	 Job *r=new KilledJob();
-	 r->parent=j->parent;
-	 r->cmdline=j->cmdline;
-	 j->cmdline=0;
-	 j->parent->ReplaceWaiting(j,r);
-      }
-      assert(FindWhoWaitsFor(j)==0);
-      Delete(j);
-   }
+      Kill(j);
 }
 
 void Job::KillAll()
 {
-   for(Job *scan=chain; scan; )
-   {
-      Job *tmp=scan;
-      scan=scan->next;
-      if(tmp->jobno>=0)
-	 Delete(tmp);
-   }
+   for(Job *scan=chain; scan; scan=scan->next)
+      if(scan->jobno>=0)
+	 Job::Kill(scan);
+   CollectGarbage();
+}
+void Job::Cleanup()
+{
+   for(Job *scan=chain; scan; scan=scan->next)
+      Job::Kill(scan);
+   CollectGarbage();
 }
 
 void  Job::SendSig(int n,int sig)
@@ -334,7 +339,7 @@ void  Job::ListDoneJobs()
    for(Job *scan=chain; scan; scan=scan->next)
    {
       if(scan->jobno>=0 && (scan->parent==this || scan->parent==0)
-         && scan->Done())
+         && !scan->deleting && scan->Done())
       {
 	 fprintf(f,_("[%d] Done (%s)"),scan->jobno,
 	    scan->cmdline?scan->cmdline:"?");
@@ -350,17 +355,13 @@ void  Job::ListDoneJobs()
 
 void  Job::BuryDoneJobs()
 {
-   for(Job *scan=chain; scan; )
+   for(Job *scan=chain; scan; scan=scan->next)
    {
       if((scan->parent==this || scan->parent==0) && scan->jobno>=0
-		  && scan->Done())
-      {
-	 Delete(scan);
-      	 scan=chain;
-      }
-      else
-	 scan=scan->next;
+		  && !scan->deleting && scan->Done())
+	 scan->deleting=true;
    }
+   CollectGarbage();
 }
 
 void Job::fprintf(FILE *file,const char *fmt,...)
@@ -549,3 +550,14 @@ bool Job::CheckForWaitLoop(Job *parent)
 	 return true;
    return false;
 }
+
+void Job::WaitDone()
+{
+   for(;;)
+   {
+      SMTask::Schedule();
+      if(Done())
+	 break;
+      SMTask::Block();
+   }
+}
