@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LsCache.cc,v 1.31 2005/12/15 08:50:40 lav Exp $ */
+/* $Id: LsCache.cc,v 1.34 2006/03/15 11:16:09 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -322,9 +322,11 @@ void LsCache::SetDirectory(FileAccess *p_loc, const char *path, bool dir)
       return;
 
    FileAccess::Path origdir = p_loc->GetCwd();
+   FileAccess::Path new_cwd = origdir;
 
-   p_loc->Chdir(path,false);
+   new_cwd.Change(path,!dir);
    const char *entry = dir? "1":"0";
+   p_loc->SetCwd(new_cwd);
    LsCache::Add(p_loc,"",FileAccess::CHANGE_DIR, dir?FA::OK:FA::NO_FILE, entry, strlen(entry));
    p_loc->SetCwd(origdir);
 }
@@ -336,7 +338,9 @@ void LsCache::SetDirectory(FileAccess *p_loc, const char *path, bool dir)
 int LsCache::IsDirectory(FileAccess *p_loc,const char *dir_c)
 {
    FileAccess::Path origdir = p_loc->GetCwd();
-   p_loc->Chdir(dir_c, false);
+   FileAccess::Path new_cwd = origdir;
+   new_cwd.Change(dir_c);
+   p_loc->SetCwd(new_cwd);
 
    int ret = -1;
 
@@ -376,10 +380,8 @@ int LsCache::IsDirectory(FileAccess *p_loc,const char *dir_c)
    /* We know this is a file or a directory if the dirname is cached and
     * contains the basename. */
    {
-      p_loc->SetCwd(origdir);
-      char *dir = alloca_strdup(dir_c);
-      p_loc->Chdir(dirname_modify(dir), false);
-
+      new_cwd.Change("..");
+      p_loc->SetCwd(new_cwd);
 
       const FileSet *fs=FindFileSet(p_loc, "", FA::MP_LIST);
       if(!fs)
