@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.126 2005/08/05 08:54:22 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.133 2006/04/19 09:48:04 lav Exp $ */
 
 #include <config.h>
 
@@ -35,7 +35,6 @@
 #include "ChmodJob.h"
 #include "misc.h"
 #include "plural.h"
-#include "getopt.h"
 #include "FindJob.h"
 #include "url.h"
 #include "CopyJob.h"
@@ -379,6 +378,8 @@ void  MirrorJob::HandleFile(FileInfo *file)
 
 	 mj->verbose_report=verbose_report;
 	 mj->newer_than=newer_than;
+	 mj->older_than=older_than;
+	 mj->size_range=size_range;
 	 mj->parallel=parallel;
 	 mj->pget_n=pget_n;
 	 mj->pget_minchunk=pget_minchunk;
@@ -391,6 +392,8 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 mj->script_name=xstrdup(script_name);
 	 mj->script_only=script_only;
 
+	 mj->max_error_count=max_error_count;
+
 	 if(verbose_report>=3)
 	    Report(_("Mirroring directory `%s'"),mj->target_relative_dir);
 
@@ -484,6 +487,10 @@ void  MirrorJob::InitSets(FileSet *source,FileSet *dest)
 
    if(newer_than!=(time_t)-1)
       to_transfer->SubtractOlderThan(newer_than);
+   if(older_than!=(time_t)-1)
+      to_transfer->SubtractNewerThan(older_than);
+   if(size_range)
+      to_transfer->SubtractSizeOutside(size_range);
 
    if(flags&NO_RECURSION)
       to_transfer->SubtractDirs();
@@ -763,6 +770,8 @@ int   MirrorJob::Do()
 	 transfer_count--;
 	 m=MOVED;
       }
+      if(max_error_count>0 && stats.error_count>=max_error_count)
+	 goto pre_FINISHING;
       while(transfer_count<parallel && state==WAITING_FOR_TRANSFER)
       {
 	 file=to_transfer->curr();
@@ -794,6 +803,8 @@ int   MirrorJob::Do()
 	 transfer_count--;
 	 m=MOVED;
       }
+      if(max_error_count>0 && stats.error_count>=max_error_count)
+	 goto pre_FINISHING;
       while(transfer_count<parallel && (state==TARGET_REMOVE_OLD || state==TARGET_REMOVE_OLD_FIRST))
       {
 	 file=0;
@@ -832,7 +843,12 @@ int   MirrorJob::Do()
 	 {
 	    ArgV args("rm");
 	    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
-	       args.Append("-r");
+	    {
+	       if(flags&NO_RECURSION)
+		  args.setarg(0,"rmdir");
+	       else
+		  args.Append("-r");
+	    }
 	    args.Append(target_session->GetFileURL(file->name));
 	    char *cmd=args.CombineQuoted();
 	    fprintf(script,"%s\n",cmd);
@@ -849,7 +865,15 @@ int   MirrorJob::Do()
 	    AddWaiting(j);
 	    transfer_count++;
 	    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
-	       j->Recurse();
+	    {
+	       if(flags&NO_RECURSION)
+	       {
+		  args->setarg(0,"rmdir");
+		  j->Rmdir();
+	       }
+	       else
+		  j->Recurse();
+	    }
 	 }
 	 if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
 	 {
@@ -880,6 +904,8 @@ int   MirrorJob::Do()
 	 transfer_count--;
 	 m=MOVED;
       }
+      if(max_error_count>0 && stats.error_count>=max_error_count)
+	 goto pre_FINISHING;
       while(transfer_count<parallel && state==TARGET_CHMOD)
       {
 	 file=to_transfer->curr();
@@ -1013,12 +1039,15 @@ MirrorJob::MirrorJob(MirrorJob *parent,
    target_list_info=0;
 
    flags=0;
+   max_error_count=0;
 
    exclude=0;
 
    set_state(INITIAL_STATE);
 
    newer_than=(time_t)-1;
+   older_than=(time_t)-1;
+   size_range=0;
 
    script_name=0;
    script=0;
@@ -1066,7 +1095,11 @@ MirrorJob::~MirrorJob()
    SessionPool::Reuse(source_session);
    SessionPool::Reuse(target_session);
    if(!parent_mirror)
+   {
+      // common pointers, delete only on top level.
       delete exclude;
+      delete size_range;
+   }
    if(script && script_needs_closing)
       fclose(script);
 }
@@ -1121,6 +1154,23 @@ void MirrorJob::SetNewerThan(const char *f)
    }
    newer_than=st.st_mtime;
 }
+void MirrorJob::SetOlderThan(const char *f)
+{
+   time_t t=now;
+   t=get_date(f,&t);
+   if(t>0)
+   {
+      older_than=t;
+      return;
+   }
+   struct stat st;
+   if(stat(f,&st)==-1)
+   {
+      perror(f);
+      return;
+   }
+   older_than=st.st_mtime;
+}
 
 mode_t MirrorJob::get_mode_mask()
 {
@@ -1223,12 +1273,16 @@ CMD(mirror)
       {"only-newer",no_argument,0,'n'},
       {"no-recursion",no_argument,0,'r'},
       {"no-perms",no_argument,0,'p'},
+      {"perms",no_argument,0,'p'+512},
       {"no-umask",no_argument,0,256+'u'},
       {"continue",no_argument,0,'c'},
       {"reverse",no_argument,0,'R'},
       {"verbose",optional_argument,0,'v'},
       {"newer-than",required_argument,0,'N'},
+      {"older-than",required_argument,0,256+'O'},
+      {"size-range",required_argument,0,512+'S'},
       {"dereference",no_argument,0,'L'},
+      {"no-dereference",no_argument,0,'L'+512},
       {"use-cache",no_argument,0,256+'C'},
       {"Remove-source-files",no_argument,0,256+'R'},
       {"parallel",optional_argument,0,'P'},
@@ -1243,11 +1297,13 @@ CMD(mirror)
       {"use-pget-n",optional_argument,0,256+'p'},
       {"no-symlinks",no_argument,0,256+'y'},
       {"loop",no_argument,0,256+'l'},
+      {"max-errors",required_argument,0,256+'E'},
       {0}
    };
 
    int opt;
    int flags=0;
+   int max_error_count=0;
 
    bool use_cache=false;
 
@@ -1256,6 +1312,8 @@ CMD(mirror)
 
    int	 verbose=0;
    const char *newer_than=0;
+   const char *older_than=0;
+   Range *size_range=0;
    bool  remove_source_files=false;
    int	 parallel=ResMgr::Query("mirror:parallel-transfer-count",0);
    int	 use_pget=ResMgr::Query("mirror:use-pget-n",0);
@@ -1268,6 +1326,8 @@ CMD(mirror)
 
    if(!ResMgr::QueryBool("mirror:set-permissions",0))
       flags|=MirrorJob::NO_PERMS;
+   if(ResMgr::QueryBool("mirror:dereference",0))
+      flags|=MirrorJob::RETR_SYMLINKS;
 
    args->rewind();
    while((opt=args->getopt_long("esi:x:I:X:nrpcRvN:LPa",mirror_opts,0))!=EOF)
@@ -1295,6 +1355,9 @@ CMD(mirror)
       case('p'):
 	 flags|=MirrorJob::NO_PERMS;
 	 break;
+      case('p'+512):
+	 flags&=~MirrorJob::NO_PERMS;
+	 break;
       case('c'):
 	 flags|=MirrorJob::CONTINUE;
 	 break;
@@ -1345,6 +1408,9 @@ CMD(mirror)
       case('L'):
 	 flags|=MirrorJob::RETR_SYMLINKS;
 	 break;
+      case('L'+512):
+	 flags&=~MirrorJob::RETR_SYMLINKS;
+	 break;
       case('v'):
 	 if(optarg)
 	    verbose=atoi(optarg);
@@ -1356,6 +1422,18 @@ CMD(mirror)
       case('N'):
 	 newer_than=optarg;
 	 break;
+      case(256+'O'):
+	 older_than=optarg;
+	 break;
+      case(512+'S'):
+	 size_range=new Range(optarg);
+	 if(size_range->Error())
+	 {
+	    eprintf("%s: --size-range \"%s\": %s\n",
+	       args->a0(),optarg,size_range->ErrorText());
+	    goto no_job;
+	 }
+	 break;
       case(256+'u'):
 	 flags|=MirrorJob::NO_UMASK;
 	 break;
@@ -1402,10 +1480,14 @@ CMD(mirror)
       case(256+'l'):
 	 flags|=MirrorJob::LOOP;
 	 break;
+      case(256+'E'):
+	 max_error_count=atoi(optarg);
+	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
       no_job:
 	 delete exclude;
+	 delete size_range;
 	 if(source_session)
 	    SMTask::Delete(source_session);
 	 if(target_session)
@@ -1458,7 +1540,7 @@ CMD(mirror)
 	    }
 	    target_dir=alloca_strdup(target_url.path);
 	 }
-	 if(last_char(target_dir)=='/' && basename_ptr(target_dir)[0]!='/')
+	 if(last_char(arg)=='/' && basename_ptr(arg)[0]!='/')
 	 {
 	    // user wants source dir name appended.
 	    const char *base=basename_ptr(source_dir);
@@ -1502,6 +1584,10 @@ CMD(mirror)
 
    if(newer_than)
       j->SetNewerThan(newer_than);
+   if(older_than)
+      j->SetOlderThan(older_than);
+   if(size_range)
+      j->SetSizeRange(size_range);
    j->UseCache(use_cache);
    if(remove_source_files)
       j->RemoveSourceFiles();
@@ -1531,6 +1617,7 @@ CMD(mirror)
       if(!script_file)
 	 j->SetScriptFile("-");
    }
+   j->SetMaxErrorCount(max_error_count);
 
    return j;
 
