@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.h,v 1.35 2004/10/14 12:02:36 lav Exp $ */
+/* $Id: NetAccess.h,v 1.37 2006/02/06 10:57:27 lav Exp $ */
 
 #ifndef NETACCESS_H
 #define NETACCESS_H
@@ -64,6 +64,8 @@ protected:
    void	 SetSocketMaxseg(int sock);
 
    static void KeepAlive(int sock);
+   static void MinimizeLatency(int sock);
+   static void MaximizeThroughput(int sock);
    static void ReuseAddress(int sock);
    static void SetSocketBuffer(int sock,int val);
    static void SetSocketMaxseg(int sock,int val);
