@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ProcWait.h,v 1.5 2002/06/18 16:14:30 lav Exp $ */
+/* $Id: ProcWait.h,v 1.8 2005/06/21 12:12:06 lav Exp $ */
 
 #ifndef PROCWAIT_H
 #define PROCWAIT_H
@@ -53,7 +53,7 @@ protected:
 
 public:
    int	 Do();
-   int	 GetState() { return status; }
+   State GetState() { return status; }
    int	 GetInfo() { return term_info; }
    int	 Kill(int sig=SIGTERM);
 
