@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SMTask.cc,v 1.36 2005/06/22 08:04:21 lav Exp $ */
+/* $Id: SMTask.cc,v 1.41 2006/02/16 08:03:32 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -60,6 +60,7 @@ SMTask::SMTask()
    suspended=false;
    suspended_slave=false;
    running=0;
+   ref_count=0;
    deleting=false;
    task_count++;
 #ifdef TASK_DEBUG
@@ -107,6 +108,7 @@ SMTask::~SMTask()
 #endif
    task_count--;
    assert(!running);
+   assert(!ref_count);
    // remove from the chain
    SMTask **scan=&chain;
    while(*scan)
@@ -114,22 +116,36 @@ SMTask::~SMTask()
       if(*scan==this)
       {
 	 *scan=next;
-	 break;
+	 return;
       }
       scan=&((*scan)->next);
    }
+   assert(!"SMTask dtor called with unregistered task");
 }
 
 void SMTask::Delete(SMTask *task)
 {
    if(!task)
       return;
-   if(task->deleting)
-      return;
    task->deleting=true;
-   if(!task->running)
+   if(!task->running && !task->ref_count)
       delete task;
 }
+void SMTask::_DeleteRef(SMTask *task)
+{
+   if(!task)
+      return;
+   assert(task->ref_count>0);
+   task->ref_count--;
+   Delete(task);
+   task=0;
+}
+void SMTask::_MakeRef(SMTask *task)
+{
+   if(!task)
+      return;
+   task->ref_count++;
+}
 
 void SMTask::Enter(SMTask *task)
 {
@@ -166,6 +182,38 @@ void SMTask::RollAll(int max_time)
 	 && (max_time==0 || now<time_limit));
 }
 
+int SMTask::CollectGarbage()
+{
+   int count=0;
+   bool repeat_gc;
+   do {
+      repeat_gc=false;
+      SMTask *scan=chain;
+      while(scan)
+      {
+	 if(scan->running || !scan->deleting || scan->ref_count)
+	 {
+	    scan=scan->next;
+	    continue;
+	 }
+	 repeat_gc=true;
+	 count++;
+	 if(scan->next)
+	 {
+	    Enter(scan->next); // protect it from deleting
+	    delete scan;
+	    Leave(scan=current);
+	 }
+	 else
+	 {
+	    delete scan;
+	    break;
+	 }
+      }
+   } while(repeat_gc);
+   return count;
+}
+
 void SMTask::Schedule()
 {
    SMTask *scan;
@@ -189,32 +237,20 @@ void SMTask::Schedule()
 	 scan=scan->next;
 	 continue;
       }
+
       if(repeat)
 	 scan->block.SetTimeout(0); // little optimization
 
-      int res=STALL;
+      Enter(scan);	   // mark it current and running.
+      int res=scan->Do();  // let it run.
+      Leave(scan);	   // unmark it running and change current.
 
-      Enter(scan);	// mark it current and running.
-      SMTask *entered=current;
-      if(!current->deleting)
-	 res=current->Do(); // let it run unless it is dying.
-      assert(current==entered);
-      scan=scan->next;	// move to a next task.
-      SMTask *to_delete=0;
-      if(current->deleting)
-	 to_delete=current;
-      if(res!=MOVED && current->block.GetTimeout()==0)
-	 res=MOVED;
-#if 0
-      if(res==MOVED)
-	 printf("MOVED:%p\n",current);
-#endif
-      Leave(current);	// unmark it running and change current.
-
-      delete to_delete;
-      if(res==MOVED || to_delete)
+      if(res==MOVED || scan->block.GetTimeout()==0)
 	 repeat=true;
+      scan=scan->next;	   // move to a next task.
    }
+   if(CollectGarbage())
+      repeat=true;
    if(repeat)
    {
       sched_total.SetTimeout(0);
@@ -239,17 +275,6 @@ void SMTask::ReconfigAll(const char *name)
       scan->Reconfig(name);
    sched_total.SetTimeout(0);  // for new values handling
 }
-void SMTask::DeleteAll()
-{
-   SMTask **scan=&chain;
-   while(*scan)
-   {
-      SMTask *old=*scan;
-      Delete(*scan);
-      if(*scan==old)
-	 scan=&(*scan)->next;
-   }
-}
 
 int SMTaskInit::Do()
 {
@@ -274,6 +299,7 @@ int SMTask::TaskCount()
 
 void SMTask::Cleanup()
 {
+  CollectGarbage();
   delete init_task;
 }
 
