@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,8 +39,12 @@ class SMTask
    bool	 suspended;
    bool	 suspended_slave;
 
+   static void _MakeRef(SMTask*);
+   static void _DeleteRef(SMTask*);
+
 protected:
    int	 running;
+   int	 ref_count;
    bool	 deleting;
 
    enum
@@ -67,6 +71,7 @@ public:
    static void UpdateNow() { now.SetToCurrentTime(); }
 
    static void Schedule();
+   static int CollectGarbage();
    static void Block() { sched_total.Block(); }
 
    void Suspend();
@@ -86,9 +91,11 @@ public:
 
    SMTask();
 
+   void DeleteLater() { deleting=true; }
    static void Delete(SMTask *);
+   template<class T> static void DeleteRef(T *&task) { _DeleteRef(task); task=0; }
+   template<class T> static T *MakeRef(T *task) { _MakeRef(task); return task; }
    static int Roll(SMTask *);
-   static void DeleteAll();
    static void RollAll(int max_time);
 
    static SMTask *current;
