@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SleepJob.cc,v 1.19 2002/07/31 11:28:01 lav Exp $ */
+/* $Id: SleepJob.cc,v 1.22 2006/02/06 11:04:00 lav Exp $ */
 
 #include <config.h>
 #include <ctype.h>
@@ -131,7 +131,7 @@ void SleepJob::lftpMovesToBackground()
 #define session (parent->session)
 Job *cmd_sleep(CmdExec *parent)
 {
-   char *op=args->a0();
+   const char *op=args->a0();
    if(args->count()!=2)
    {
       eprintf(_("%s: argument required. "),op);
@@ -183,7 +183,7 @@ Job *cmd_at(CmdExec *parent)
    int date_len=0;
    for(;;)
    {
-      char *arg=args->getnext();
+      const char *arg=args->getnext();
       if(arg==0)
 	 break;
       if(!strcmp(arg,"--"))
@@ -208,7 +208,11 @@ Job *cmd_at(CmdExec *parent)
    xfree(date);
 
    if(when==0 || when==(time_t)-1)
+   {
+      const char *e=get_date_error();
+      eprintf("%s: %s\n",args->a0(),e?e:"unknown parse error");
       return 0;
+   }
 
    if(when<now)
       when+=86400;
