@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: buffer.h,v 1.37 2005/06/22 08:04:22 lav Exp $ */
+/* $Id: buffer.h,v 1.39 2006/02/06 10:57:29 lav Exp $ */
 
 #ifndef BUFFER_H
 #define BUFFER_H
@@ -145,7 +145,7 @@ public:
    void ResetTranslation();
    void Put(const char *buf,int size)
       {
-	 if(mode==PUT && backend_translate)
+	 if(mode==PUT)
 	    PutTranslated(buf,size);
 	 else
 	    Buffer::Put(buf,size);
