@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2006 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.226 2005/08/03 07:12:47 lav Exp $ */
+/* $Id: commands.cc,v 1.246 2006/04/24 10:32:00 lav Exp $ */
 
 #include <config.h>
 
@@ -65,7 +65,6 @@
 #include "bookmark.h"
 #include "log.h"
 #include "module.h"
-#include "getopt.h"
 #include "FileCopy.h"
 #include "DummyProto.h"
 #include "QueueFeeder.h"
@@ -95,7 +94,7 @@ CMD(ver);   CMD(close);  CMD(bookmark);CMD(lftp);
 CMD(echo);  CMD(suspend);CMD(sleep);   CMD(slot);
 CMD(at);    CMD(find);   CMD(command); CMD(module);
 CMD(lpwd);  CMD(glob);	 CMD(chmod);   CMD(queue);
-CMD(repeat);CMD(get1);   CMD(history); CMD(tasks);
+CMD(repeat);CMD(get1);   CMD(tasks);
 
 #ifdef MODULE_CMD_MIRROR
 # define cmd_mirror 0
@@ -190,6 +189,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	    " -S                   - sort by file size\n"
 	    " --user, --group, --perms, --date, --linkcount, --links\n"
 	    "                      - show individual fields\n"
+	    " --time-style=STYLE   - use specified time format\n"
 	    "\n"
 	    "By default, cls output is cached, to see new listing use `recls' or\n"
 	    "`cache flush'.\n"
@@ -264,13 +264,6 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -a  all types\n")},
    {"help",    cmd_help,   N_("help [<cmd>]"),
 	 N_("Print help for command <cmd>, or list of available commands\n")},
-   {"history", cmd_history,N_("history -w file|-r file|-c|-l [cnt]"),
-	 N_(" -w <file> Write history to file.\n"
-	 " -r <file> Read history from file; appends to current history.\n"
-	 " -c  Clear the history.\n"
-	 " -l  List the history (default).\n"
-	 "Optional argument cnt specifies the number of history lines to list,\n"
-	 "or \"all\" to list all entries.\n")},
    {"jobs",    cmd_jobs,   "jobs [-v]",
 	 N_("List running jobs. -v means verbose, several -v can be specified.\n")},
    {"kill",    cmd_kill,   N_("kill all|<job_no>"),
@@ -637,12 +630,25 @@ Job *CmdExec::builtin_cd()
 Job *CmdExec::builtin_exit()
 {
    bool bg=false;
+   bool kill=false;
    int code=prev_exit_code;
-   if(args->count()>=2)
+   CmdExec *exec=this;
+   const char *a;
+   args->rewind();
+   while((a=args->getnext())!=0)
    {
-      const char *a=args->getarg(1);
       if(!strcmp(a,"bg"))
 	 bg=true;
+      if(!strcmp(a,"top") || !strcmp(a,"bg"))
+      {
+	 if(top)
+	    exec=top;
+      }
+      else if(!strcmp(a,"kill"))
+      {
+	 kill=true;
+	 bg=false;
+      }
       else if(sscanf(a,"%i",&code)!=1)
       {
 	 eprintf(_("Usage: %s [<exit_code>]\n"),args->a0());
@@ -650,7 +656,7 @@ Job *CmdExec::builtin_exit()
       }
    }
    // Note: one job is this CmdExec.
-   if(!bg && top_level
+   if(!bg && exec->top_level
    && !ResMgr::QueryBool("cmd:move-background",0) && NumberOfJobs()>1)
    {
       eprintf(_(
@@ -659,11 +665,26 @@ Job *CmdExec::builtin_exit()
       ));
       return 0;
    }
-   while(!Done())
-      RemoveFeeder();
-   exit_code=prev_exit_code=code;
+   if(kill)
+      Job::KillAll();
+   exec->Exit(code);
+   exit_code=code;
    return 0;
 }
+void CmdExec::Exit(int code)
+{
+   while(feeder)
+      RemoveFeeder();
+   if(interactive)
+   {
+      ListDoneJobs();
+      BuryDoneJobs();
+      if(FindJob(last_bg)==0)
+	 last_bg=-1;
+   }
+   exit_code=prev_exit_code=code;
+   return;
+}
 
 CmdFeeder *lftp_feeder=0;
 Job *CmdExec::builtin_lftp()
@@ -858,30 +879,32 @@ Job *CmdExec::builtin_open()
       if(host && host[0])
       {
 	 url=new ParsedURL(host);
+	 bool no_proto=(!url->proto);
 
-	 const ParsedURL &uc=*url;
-	 if(uc.host && uc.host[0])
+	 if(no_proto && url->host)
 	 {
-	    cwd_history.Set(session,session->GetCwd());
-
-	    FileAccess *new_session=0;
-
-	    const char *p=uc.proto;
-	    if(!p)
-	       p=ResMgr::Query("cmd:default-protocol",uc.host);
+	    const char *p=ResMgr::Query("cmd:default-protocol",url->host);
 	    if(!p)
 	       p="ftp";
-	    new_session=FileAccess::New(p,uc.host);
-	    if(!new_session)
-	    {
-	       eprintf("%s: %s%s\n",args->a0(),p,
-			_(" - not supported protocol"));
-	       return 0;
-	    }
+	    char *u=string_alloca(strlen(p)+3+strlen(host)+1);
+	    sprintf(u,"%s://%s",p,host);
+	    delete url;
+	    url=new ParsedURL(u);
+	 }
+	 if(user)
+	 {
+	    url->user=user;
+	    url->pass=pass;
+	    char *host1=url->Combine();
+	    delete url;
+	    url=new ParsedURL(host1);
+	    xfree(host1);
+	 }
 
-	    saved_session=session;
-	    session=0;
-	    ChangeSession(new_session);
+	 const ParsedURL &uc=*url;
+	 if(uc.host && uc.host[0] && uc.proto)
+	 {
+	    cwd_history.Set(session,session->GetCwd());
 
 	    if(uc.user && !user)
 	       user=uc.user;
@@ -895,10 +918,22 @@ Job *CmdExec::builtin_open()
 	       port=uc.port;
 	    if(uc.path && !path)
 	       path=uc.path;
+
+	    FileAccess *new_session=FileAccess::New(uc.proto,host,port);
+	    if(!new_session)
+	    {
+	       eprintf("%s: %s%s\n",args->a0(),uc.proto,
+			_(" - not supported protocol"));
+	       return 0;
+	    }
+
+	    saved_session=session;
+	    session=0;
+	    ChangeSession(new_session);
 	 }
 
 	 // user gets substituted only if no proto is specified.
-	 if(!pass && (user || !uc.proto))
+	 if(!pass && (user || no_proto))
 	 {
 	    nrc=NetRC::LookupHost(host,user);
 	    if(nrc)
@@ -968,9 +1003,10 @@ Job *CmdExec::builtin_open()
 	 session->SetCwd(FileAccess::Path(old,is_file,url));
       }
 
-      char *s=string_alloca(strlen(path)*2+40);
+      const char *cd_arg=(url && url->orig_url)?url->orig_url:path;
+      char *s=string_alloca(strlen(cd_arg)*2+40);
       strcpy(s,"&& cd \"");
-      unquote(s+strlen(s),(url && url->orig_url)?url->orig_url:path);
+      unquote(s+strlen(s),cd_arg);
       strcat(s,"\"");
       if(background)
 	 strcat(s,"&");
@@ -1085,7 +1121,7 @@ Job *CmdExec::builtin_queue()
 	  * it. */
 	 if(!isdigit((unsigned char)optarg[0]) || atoi(optarg) == 0)
 	 {
-	    eprintf(_("%s: -n: Number expected. "), args->a0());
+	    eprintf(_("%s: -n: positive number expected. "), args->a0());
 	    goto err;
 	 }
 	 /* make offsets match the jobs output (starting at 1) */
@@ -1142,7 +1178,7 @@ Job *CmdExec::builtin_queue()
 	       queue->Suspend();
 	    }
 	    else
-	       queue->PrintStatus(0,"");
+	       queue->PrintStatus(2,"");
 	    exit_code=0;
 	    break;
 	 }
@@ -1380,6 +1416,7 @@ const char *FileSetOutput::parse_argv(ArgV *a)
       {"nocase", no_argument,0,'i'},
       {"sortnocase", no_argument,0,'I'},
       {"dirsfirst", no_argument,0,'D'},
+      {"time-style", required_argument,0,256+'T'},
 
       {"sort", required_argument,0, 0},
       {"reverse",no_argument,0,'r'},
@@ -1392,6 +1429,8 @@ const char *FileSetOutput::parse_argv(ArgV *a)
       {0,0,0,0}
    };
 
+   const char *time_style=ResMgr::Query("cmd:time-style",0);
+
    int opt, longopt;
    while((opt=a->getopt_long(":1BdFhiklqsDISrt", cls_options, &longopt))!=EOF)
    {
@@ -1470,6 +1509,9 @@ const char *FileSetOutput::parse_argv(ArgV *a)
       case('r'):
 	 sort_reverse = true;
 	 break;
+      case(256+'T'):
+	 time_style=optarg;
+	 break;
 
       default:
 	 /* silly getopt won't give us its error instead of printing it, oh well.
@@ -1483,6 +1525,35 @@ const char *FileSetOutput::parse_argv(ArgV *a)
       a->delarg(1);
    a->rewind();
 
+   xfree(time_fmt); time_fmt=0;
+   if(time_style && time_style[0]) {
+      if(time_style[0]=='+')
+	 time_fmt=xstrdup(time_style+1);
+      else if(!strcmp(time_style,"full-iso"))
+//	 time_fmt=xstrdup("%Y-%m-%d %H:%M:%S.%N %z"); // %N and %z are GNU extensions
+	 time_fmt=xstrdup("%Y-%m-%d %H:%M:%S");
+      else if(!strcmp(time_style,"long-iso"))
+	 time_fmt=xstrdup("%Y-%m-%d %H:%M");
+      else if(!strcmp(time_style,"iso"))
+	 time_fmt=xstrdup("%Y-%m-%d \n%m-%d %H:%M");
+      else
+	 time_fmt=xstrdup(time_style);
+      need_exact_time=false;
+      if(time_fmt) {
+	 static const char exact_fmts[][3]={"%H","%M","%S","%N",""};
+	 int sep=strcspn(time_fmt,"\n|");
+	 for(int i=0; exact_fmts[i][0]; i++) {
+	    const char *f=strstr(time_fmt,exact_fmts[i]);
+	    if(!f)
+	       continue;
+	    if(i>1 || sep>f-time_fmt) {
+	       need_exact_time=true;
+	       break;
+	    }
+	 }
+      }
+   }
+
    return NULL;
 }
 
@@ -1581,7 +1652,7 @@ CMD(get)
    bool glob=false;
    bool make_dirs=false;
    bool reverse=false;
-   char *output_dir=0;
+   const char *output_dir=0;
 
    if(!strncmp(op,"re",2))
    {
@@ -1608,6 +1679,12 @@ CMD(get)
       opts="cEeadO:";
       reverse=true;
    }
+   if(!reverse)
+   {
+      output_dir=ResMgr::Query("xfer:destination-directory",session->GetHostName());
+      if(!*output_dir)
+	 output_dir=0;
+   }
    while((opt=args->getopt(opts))!=EOF)
    {
       switch(opt)
@@ -1655,7 +1732,8 @@ CMD(get)
 	 goto err;
       }
       delete get_args;
-      output_dir=xstrdup(output_dir);  // save it from deleting.
+      char *output_dir_dup=xstrdup(output_dir);  // save it from deleting.
+      output_dir=0;
       // remove options
       while(args->getindex()>1)
 	 args->delarg(1);
@@ -1666,8 +1744,8 @@ CMD(get)
 	 j->DeleteFiles();
       if(ascii)
 	 j->Ascii();
-      if(output_dir)
-	 j->OutputDir(output_dir);
+      if(output_dir_dup)
+	 j->OutputDir(output_dir_dup);
       args=0;
       return j;
    }
@@ -1770,12 +1848,12 @@ CMD(rm)
    if(args->getcurr()==0)
       goto print_usage;
 
-   rmJob *j=(rmdir?
-	     new rmdirJob(session->Clone(),args):
-	     new rmJob(session->Clone(),args));
+   rmJob *j=new rmJob(session->Clone(),args);
 
    if(recursive)
       j->Recurse();
+   if(rmdir)
+      j->Rmdir();
 
    args=0;
 
@@ -2476,7 +2554,7 @@ CMD(help)
 CMD(ver)
 {
    printf(
-      _("LFTP | Version %s | Copyright (c) 1996-2005 Alexander V. Lukyanov\n"),VERSION);
+      _("LFTP | Version %s | Copyright (c) 1996-2006 Alexander V. Lukyanov\n"),VERSION);
    printf("\n");
    printf(
       _("LFTP is free software, covered by the GNU General Public License, and you are\n"
@@ -2494,11 +2572,13 @@ CMD(ver)
    {
       const char *lib_name;
       const char *symbol;
-      enum type_t { STRING_PTR, FUNC0 } type;
+      enum type_t { STRING_PTR, FUNC0, INT8_8 } type;
       const char *skip_prefix;
       typedef const char *(*func0)(int);
       const char *query() const
 	 {
+	    static char ver[10];
+	    int v;
 	    void *sym_ptr=dlsym(RTLD_DEFAULT,symbol);
 	    if(!sym_ptr)
 	       return 0;
@@ -2511,6 +2591,10 @@ CMD(ver)
 	    case FUNC0:
 	       str=((func0)sym_ptr)(0);
 	       break;
+	    case INT8_8:
+	       v=*(int*)sym_ptr;
+	       sprintf(ver,"%d.%d",(v>>8)&255,v&255);
+	       str=ver;
 	    }
 	    if(!str)
 	       return 0;
@@ -2525,6 +2609,7 @@ CMD(ver)
       {"Expat",	     "XML_ExpatVersion",     VersionInfo::FUNC0,     "expat_"},
       {"OpenSSL",    "SSL_version_str",	     VersionInfo::STRING_PTR,"OpenSSL "},
       {"GnuTLS",     "gnutls_check_version", VersionInfo::FUNC0,     0},
+      {"libiconv",   "_libiconv_version",    VersionInfo::INT8_8,    0},
       {0}
    };
 
@@ -2652,7 +2737,7 @@ CMD(bookmark)
    else if(!strcasecmp(op,"edit"))
    {
       lftp_bookmarks.Remove(""); // force bookmark file creation
-      parent->PrependCmd("shell \"/bin/sh -c 'exec ${EDITOR:-vi} $HOME/.lftp/bookmarks'\"\n");
+      parent->PrependCmd("shell \"/bin/sh -c 'exec ${EDITOR:-vi} ${LFTP_HOME:-$HOME/.lftp}/bookmarks'\"\n");
    }
    else if(!strcasecmp(op,"import"))
    {
@@ -3174,88 +3259,13 @@ CMD(get1)
    return new CopyJob(c,src,args->a0());
 }
 
-CMD(history)
-{
-   enum { READ, WRITE, CLEAR, LIST } mode = LIST;
-   const char *fn = NULL;
-   static struct option history_options[]=
-   {
-      {"read",required_argument,0,'r'},
-      {"write",required_argument,0,'w'},
-      {"clear",no_argument,0,'c'},
-      {"list",required_argument,0,'l'},
-      {0,0,0,0}
-   };
-
-   exit_code=0;
-   int opt;
-   while((opt=args->getopt_long("+r:w:cl",history_options,0))!=EOF) {
-      switch(opt) {
-      case 'r':
-	 mode = READ;
-	 fn = optarg;
-	 break;
-      case 'w':
-	 mode = WRITE;
-	 fn = optarg;
-	 break;
-      case 'c':
-	 mode = CLEAR;
-	 break;
-      case 'l':
-	 mode = LIST;
-	 break;
-      case '?':
-	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
-	 return 0;
-      }
-   }
-
-   int cnt = 16;
-   if(const char *arg = args->getcurr()) {
-      if(!strcasecmp(arg, "all"))
-	 cnt = -1;
-      else if(isdigit(arg[0]))
-	 cnt = atoi(arg);
-      else {
-	 eprintf(_("%s: %s - not a number\n"), args->a0(), args->getcurr());
-	 exit_code=1;
-	 return 0;
-      }
-   }
-
-   switch(mode) {
-   case READ:
-      if(int err = lftp_history_read(fn)) {
-	 eprintf("%s: %s: %s\n", args->a0(), fn, strerror(err));
-	 exit_code=1;
-      }
-      break;
-
-   case WRITE:
-      if(int err = lftp_history_write(fn)) {
-	 eprintf("%s: %s: %s\n", args->a0(), fn, strerror(err));
-	 exit_code=1;
-      }
-      break;
-
-   case LIST:
-      lftp_history_list(cnt);
-      break;
-   case CLEAR:
-      lftp_history_clear();
-      break;
-   }
-
-   return 0;
-}
-
 CMD(slot)
 {
    const char *n=args->getarg(1);
    if(n)
    {
       parent->ChangeSlot(n);
+      exit_code=0;
       return 0;
    }
    else
