@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1999-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.123 2005/12/27 12:11:33 lav Exp $ */
+/* $Id: resource.cc,v 1.128 2006/04/19 09:48:09 lav Exp $ */
 
 #include <config.h>
 
@@ -265,7 +265,7 @@ static ResDecl
    ResDecl24d("http:cache-control",	  "",	   0,0),
    ResDecl25 ("http:proxy",		  "",	   HttpProxyValidate,0),
    ResDecl25a("http:use-mkcol",		  "yes",   ResMgr::BoolValidate,0),
-   ResDecl25b("http:use-propfind",	  "yes",   ResMgr::BoolValidate,0),
+   ResDecl25b("http:use-propfind",	  "no",    ResMgr::BoolValidate,0),
    ResDecl26 ("http:user-agent",	  PACKAGE"/"VERSION,0,0),
    ResDecl27 ("http:cookie",		  "",	   0,0),
    ResDecl28 ("http:set-cookies",	  "no",	   0,0),
@@ -301,7 +301,8 @@ static ResDecl
    ResDecl48 ("mirror:parallel-transfer-count", "1", ResMgr::UNumberValidate,ResMgr::NoClosure),
    ResDecl49 ("mirror:exclude-regex",	  "(^|/)(\\.in\\.|\\.nfs)",ResMgr::ERegExpValidate,ResMgr::NoClosure),
    ResDecl50 ("mirror:use-pget-n",	  "1",	   ResMgr::UNumberValidate,ResMgr::NoClosure),
-   ResDecl51 ("mirror:set-permissions",	  "yes",   ResMgr::BoolValidate,ResMgr::NoClosure);
+   ResDecl51 ("mirror:set-permissions",	  "yes",   ResMgr::BoolValidate,ResMgr::NoClosure),
+   ResDecl52 ("mirror:dereference",	  "no",    ResMgr::BoolValidate,ResMgr::NoClosure);
 
 static ResDecl
    res_sftp_packets  ("sftp:max-packets-in-flight","16",ResMgr::UNumberValidate,0),
@@ -360,8 +361,12 @@ static ResDecl
 # include <langinfo.h>
 #endif
 
+bool ResMgr::class_inited;
 void ResMgr::ClassInit()
 {
+   if(class_inited)
+      return;
+   class_inited=true;
    for(ResDecl *scan=type_chain; scan; scan=scan->next)
    {
       if(scan->defvalue && scan->val_valid)
@@ -430,5 +435,9 @@ void ResMgr::ClassInit()
       Set("file:charset",0,cs);
 #endif
 
+   const char *time_style=getenv("TIME_STYLE");
+   if(time_style && *time_style)
+      Set("cmd:time-style",0,time_style);
+
    Set("xfer:verify-command",0,PKGDATADIR"/verify-file");
 }
